@@ -412,7 +412,6 @@ static char*
 cnames[] =
 {
 	[C_ADDR]	= "C_ADDR",
-	[C_BCON]	= "C_BCON",
 	[C_FAUTO]	= "C_FAUTO",
 	[C_ZFCON]	= "C_SFCON",
 	[C_SFCON]	= "C_SFCON",
@@ -420,11 +419,7 @@ cnames[] =
 	[C_FCR]		= "C_FCR",
 	[C_FOREG]	= "C_FOREG",
 	[C_FREG]	= "C_FREG",
-	[C_GACON]	= "C_GACON",
-	[C_GBRA]	= "C_GBRA",
-	[C_GCON]	= "C_GCON",
 	[C_GOK]		= "C_GOK",
-	[C_GOREG]	= "C_GOREG",
 	[C_HAUTO]	= "C_HAUTO",
 	[C_HFAUTO]	= "C_HFAUTO",
 	[C_HFOREG]	= "C_HFOREG",
@@ -437,7 +432,6 @@ cnames[] =
 	[C_LOREG]	= "C_LOREG",
 	[C_NCON]	= "C_NCON",
 	[C_NONE]	= "C_NONE",
-	[C_OFFPC]	= "C_OFFPC",
 	[C_PC]		= "C_PC",
 	[C_PSR]		= "C_PSR",
 	[C_RACON]	= "C_RACON",
@@ -445,7 +439,6 @@ cnames[] =
 	[C_REG]		= "C_REG",
 	[C_REGREG]	= "C_REGREG",
 	[C_ROREG]	= "C_ROREG",
-	[C_SACON]	= "C_SACON",
 	[C_SAUTO]	= "C_SAUTO",
 	[C_SBRA]	= "C_SBRA",
 	[C_SCON]	= "C_SCON",
